﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.ExperimentManager.Jobs;
using System.Threading;
using System.Threading.Tasks;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Clients.Common;

namespace HeuristicLab.Hive.ExperimentManager {
  public class HiveJobDownloader {
    private IEnumerable<Guid> jobIds;
    private List<Task<HiveJob>> tasks;

    public bool IsFinished {
      get {
        return tasks.TrueForAll(t => t.Status == TaskStatus.RanToCompletion ||
                                     t.Status == TaskStatus.Faulted ||
                                     t.Status == TaskStatus.Canceled);
      }
    }

    public int FinishedCount {
      get {
        return tasks.Count(t => t.Status == TaskStatus.RanToCompletion ||
                                t.Status == TaskStatus.Faulted ||
                                t.Status == TaskStatus.Canceled);
      }
    }

    public IDictionary<Guid, HiveJob> Results {
      get {
        var results = new Dictionary<Guid, HiveJob>();
        foreach(var t in tasks) {
          results.Add(t.Result.JobDto.Id, t.Result);
        }
        return results;
      }
    }

    public HiveJobDownloader(IEnumerable<Guid> jobIds) {
      this.jobIds = jobIds;
    }

    
    public void StartAsync() {
      tasks = new List<Task<HiveJob>>();
      foreach (Guid jobId in jobIds) {
        tasks.Add(Task<SerializedJob>.Factory.StartNew(
          (x) => DownloadJob(x), jobId)
          .ContinueWith((x) => DeserializeJob(x.Result)));
      }
    }
    // use semaphore to ensure only few tasks download concurrently (so not too many connections are open)
    private Semaphore taskSemaphore = new Semaphore(2, 2);
    protected SerializedJob DownloadJob(object jobId) {
      taskSemaphore.WaitOne(Timeout.Infinite);
      using (Disposable<IClientFacade> service = ServiceLocator.Instance.StreamedClientFacadePool.GetService()) {
        return service.Obj.GetLastSerializedResult((Guid)jobId).Obj;
      }
    }

    protected HiveJob DeserializeJob(SerializedJob serializedJob) {
      HiveJob job = new HiveJob(serializedJob, false);
      job.Job.Prepare(); // reduce memory consumption.
      taskSemaphore.Release();
      job.JobDto = serializedJob.JobInfo;
      return job;
    }
  }
}
